//
//  WordlistGenerator.m
//  iOS Hotspot Cracker
//
//  Copyright (c) 2013, Andreas Kurtz & Markus Trossbach, Friedrich-Alexander-University, Erlangen, Germany.
//  All rights reserved.
//

#import "WordlistGenerator.h"
#import "NSFileManager+Cache.h"

#define FILE_COUNTED_SET @"counted_set.data"
#define FILE_SORTED_ARRAY @"sorted_array.data"

@interface UITextChecker (Private)
- (id)suggestWordInLanguage:(id)lang;
@end

@interface WordlistGenerator() {
@private
    NSCountedSet* _set;
}
@end

NSInteger countedSort(id obj1, id obj2, void *context) {
    NSCountedSet *countedSet = (__bridge NSCountedSet *)(context);
    NSUInteger obj1Count = [countedSet countForObject:obj1];
    NSUInteger obj2Count = [countedSet countForObject:obj2];
    
    if (obj1Count > obj2Count) return NSOrderedAscending;
    else if (obj1Count < obj2Count) return NSOrderedDescending;
    return NSOrderedSame;
}


@implementation WordlistGenerator

+ (WordlistGenerator*)sharedInstance {
    static WordlistGenerator *sharedSingleton;
    
    @synchronized(self)
    {
        if (!sharedSingleton)
            sharedSingleton = [[WordlistGenerator alloc] init];
        return sharedSingleton;
    }
}


- (id)init {
    self = [super init];
    if(self) {
        self.numberOfWords = 500000;
        [self tryToLoadFiles];
    }
    return self;
}

- (void)generateWordlist {
    NSLog(@"Starting wordlist generator...");
    self.stage = kWordlistGenStageGeneratingWords;
    
    _set = nil;
    self.sortedWordlist = nil;
    
    NSInteger count = self.numberOfWords;
    UITextChecker* checker = [[UITextChecker alloc] init];
    NSCountedSet* set = [[NSCountedSet alloc] initWithCapacity:1842];
    double step = 0.99f / (double)count;
    
    for(NSInteger i = 0; i < count; i++) {
        NSString* str = [checker suggestWordInLanguage:@"en_US"];
        if(str) {
            [set addObject:str];
        }
        if(i % 1000 == 0){
            self.progress = step * (double)i;
        }
    }

    self.stage = kWordlistGenStageFinalizing;
    NSArray *distinctArray = [[set allObjects]
                              sortedArrayUsingFunction:countedSort context:(__bridge void *)(set)];
    
    _set = set;
    self.sortedWordlist = distinctArray;
    [self saveToFiles];
    self.stage = kWordlistGenStageAvailable;
    NSLog(@"Wordlist successfully generated, it consists of %i unique words derived from %i samples.", [distinctArray count], count);
}


- (void)start {
    [self performSelectorInBackground:@selector(generateWordlist) withObject:nil];
}

- (NSData*)dataForWordlist {
    NSMutableString* string = [[NSMutableString alloc] init];
    for(NSString* entry in _sortedWordlist) {
        [string appendFormat:@"%@\n", entry];
    }
    
    return [string dataUsingEncoding:NSUTF8StringEncoding];
}

- (void)saveToFiles {
    [NSKeyedArchiver archiveRootObject:_set toFile:[[[NSFileManager defaultManager] cacheDataPath] stringByAppendingPathComponent:FILE_COUNTED_SET]];
    [NSKeyedArchiver archiveRootObject:_sortedWordlist toFile:[[[NSFileManager defaultManager] cacheDataPath] stringByAppendingPathComponent:FILE_SORTED_ARRAY]];
}

- (void)tryToLoadFiles {
    NSString* setFile = [[[NSFileManager defaultManager] cacheDataPath] stringByAppendingPathComponent:FILE_COUNTED_SET];
    NSString* arrayFile = [[[NSFileManager defaultManager] cacheDataPath] stringByAppendingPathComponent:FILE_SORTED_ARRAY];

    NSCountedSet* set = (NSCountedSet*)[NSKeyedUnarchiver unarchiveObjectWithFile:setFile];
    NSArray* array = (NSArray*)[NSKeyedUnarchiver unarchiveObjectWithFile:arrayFile];
    
    if(set && array && [set count] == [array count] && [set count] > 1841) {
        _set = set;
        _sortedWordlist = array;
        self.stage = kWordlistGenStageAvailable;
        NSLog(@"Wordlist successfully loaded from cache files, it consists of %i unique entries.", [_sortedWordlist count]);
    } else {
        _set = nil;
        _sortedWordlist = nil;
        [[NSFileManager defaultManager] removeItemAtPath:setFile error:nil];
        [[NSFileManager defaultManager] removeItemAtPath:arrayFile error:nil];
        self.stage = kWordlistGenStageNotAvailable;
    }
    
}

@end
